{**********************************************************************
          Copyright (c) PilotLogic Software House
                   All rights reserved

 This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)
 ct9999
***********************************************************************}

// for Embedded Form Designer
(* updates for reactive title bar, and color adapt and resize bars por
  Saulo José Bnevenutti 11/2020

 *)
unit ct_emb_formeditor;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, ExtCtrls, Buttons,
  Math,
  ct_emb_basics, IDEWindowIntf, Toolwin,
  Graphics, Dialogs,
  ComCtrls, LMessages,
  LCLIntf,
  LCLType,
  LCLClasses,
  ctvarside,
  LazLoggerBase, ct_idestrconsts,
  intfgraphics,IDEImagesIntf, StdCtrls,  EditBtn, Menus, ComponentEditors,
  dateutils, types, typinfo {$IFDEF WINDOWS} , Registry, LazUTF8 {$ENDIF};

type

TEmbedHandlerOption=(ehoSendBefore,ehoSendAfter,ehoSendToForm);
TEmbedHandlerOptions=Set of TEmbedHandlerOption;

TEmbedTabSheet = class;
TEmbedFormEditor = class;

TEmbedMessageHandler = class(TPersistent)
  protected
    FHookControl: TControl;
    InheritedWndMethod: TWndMethod;
  public
    FOptions:TEmbedHandlerOptions;
    OnBeforeMessage,
    OnAfterMessage  : TEmbedMessageHandlerEvent;
    constructor Create;
    destructor Destroy; override;
    procedure HookControl(AControl: TControl);
    procedure UnHookControl;
    function  HasHookControl: Boolean;
    procedure SelfWndMethod(var TheMessage: TLMessage);
    Procedure SetDefaulOptions;
    Property TheHookControl: TControl read FHookControl;
  end;

TTEmbedFakeMainMenu = class(TToolBar)
  protected
    FMainMenu:TMainMenu;
  public
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;
    Procedure MainMenuAssign(aMainMenu:TMainMenu);
    Procedure MainMenuClear;
    Procedure MainMenuUpdate;
end;

{ TEmbedDesignPanel }

TEmbedDesignPanel = class(TPanel)
  protected
    FHasLoadedForm: Boolean;
    FLoadedForm: TCustomForm;
    FFakeWindow: TPanel;
    FFakeWindow_MouseDown: Boolean;
    FFakeWindow_MousePos: TPoint;
    FFakeWindowCaption: TPanel;
    FMessagesHandler:TEmbedMessageHandler;
    FFakeMainMenu:TTEmbedFakeMainMenu;
    FMouseArea:integer;
    FExecuteBeforeMessage: Boolean;
    FIniOK:boolean;
    OnFormLoad: TNotifyEvent;
    OnLoadedFormChangeBounds: TNotifyEvent;

    FZeroOriginPoint:Tpoint;
    FDeltaPointMousePos: TPoint;

    dsParentPage:TEmbedTabSheet;
    dsParentFormEditor: TEmbedFormEditor;
    // Modernização por Saulo J. Benvenutti
    FFakeIcon: TImage;
    FFakeBtnSystem: TPanel;
    FFakeBtnMinimize: TPanel;
    FFakeBtnMaximize: TPanel;

    FoldBorder : TBorderStyle;
    FoldBIMaximize : Boolean;
    FoldBIMinimize : Boolean;
    FoldBISysmenu  : Boolean;
    // resize guides  -----------------------------
    FvertResize : Tpanel;
    FcornerVResize : Tpanel;
    FcornerHResize : Tpanel;
    FhorizontalResize : Tpanel;
    Fdestaque : Boolean;
    Fdestaquetag : Integer;
    // ---------------------------------

    procedure InitializeWnd; override;
    procedure HideWindowDesign(aForm: TCustomForm);
    function  FormIsValid(aForm: TCustomForm): Boolean;
    function  LoadedFormIsValid: Boolean;
    function  FormZeroOrigin(aForm: TCustomForm): TPoint;
    Function  CheckForFakeMainMenu(aForm: TCustomForm):Boolean;
    Procedure UpdateFakeMainMenu;
    procedure AddForm(aForm: TCustomForm);
  public
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;
    procedure EmbeddedForm(Sender: TObject; aForm: TCustomForm);
    procedure Form_BeforeMessage(Sender: TObject; SenderForm: TControl; Msg: TLMessage);
    procedure Form_AfterMessage(Sender: TObject; SenderForm: TControl; Msg: TLMessage);
    procedure FFakeWindowMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FFakeWindowMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FFakeWindowMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FFakeWindowResize(Sender: TObject);
    procedure FFakeWindowCaptionPaint(Sender: TObject);
    Function  FindMouseArea(const X,Y:integer):integer;
    procedure UnLoadLoadedForm;
      // Modernização por Saulo J. Benvenutti
    procedure FFakeButtonsMouseEnter(Sender : Tobject);
    procedure FFakeButtonsMouseLeave(Sender : Tobject);
    procedure FFakeButtonsPaint(Sender: TObject);
    procedure FFakeButtonsDraw;
    //----------------------------------------------------
     //resizeguides
    procedure guidestResizeonMouseEnter(Sender: TObject);
    procedure guidesResizeMouseLeave(Sender: TObject);
    procedure guidesResizeMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure guidesResizeMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure guidesPaint(sender: Tobject);
    procedure PaintGuides(Sender: TObject);
    //-------------------------------------------------
    property IniOK:boolean read FIniOK write FIniOK;
    property LoadedForm: TCustomForm read FLoadedForm;
  end;

  { TEmbedFormEditor }

  TEmbedFormEditor = class(TFrame)
    ImgSystem: TImageList;
    imgWindowScreenPreview: TImage;
    LabelX: TLabel;
    LabelY: TLabel;
    LabelW: TLabel;
    LabelH: TLabel;
    pnShowHide: TPanel;
    RootPanel: TPanel;
    pnInfoPanel: TPanel;
    pnScreenPreviewBtns: TPanel;
    pnCaptionWindowScreenPreview: TPanel;
    pnScreenPreview: TPanel;
    pnWindowScreenPreview: TPanel;
    pnDesignBox: TScrollBox;
    sb_hide: TSpeedButton;
    sb_zero: TSpeedButton;
    sb_up: TSpeedButton;
    sb_bottom: TSpeedButton;
    sb_left: TSpeedButton;
    sb_right: TSpeedButton;
    sb_update: TSpeedButton;
    procedure imgWindowScreenPreviewMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure imgWindowScreenPreviewMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure imgWindowScreenPreviewMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure sb_upClick(Sender: TObject);
    procedure sb_hideClick(Sender: TObject);
  private
    FPreviewMoving: boolean;
    FPreviewMovingPoint: TPoint;
    FDesignAreaViewMoving: boolean;
    FDesignAreaViewMovingPoint: TPoint;
  public
    FDesignPanel: TEmbedDesignPanel;
    function  PixelPerc(vl, perc: single): integer;
    procedure DesignPanel_OnLoadForm(Sender: TObject);
    procedure DesignPanel_OnLoadedFormChangeBounds(Sender: TObject);
    procedure SetupPreviewFormDesign(_Form: TCustomForm);
    Procedure ActivatedDesigner;
  end;

TEmbedTabSheet = class(TTabSheet)
  private
    FOnToggleFormOrSource:TNotifyEvent;
    Function GetLoadedForm:TCustomForm;
  public
    FEditPages : TPageControl;
    FEditPageForSource : TTabSheet;
    FEditPageForDesigner : TTabSheet;

    FFormEditor: TEmbedFormEditor;
    FSynEditor : TCustomControl;  //is TIDESynEditor
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;

    procedure DoOnTabControlChange(Sender: TObject);

    procedure ShowSourceTab;
    procedure ShowFormEditorTab;

    procedure EditPages_Create;
    procedure EditPages_Free;

    procedure FormEditorsInit;
    Procedure FormEditorDelete;
    Procedure FormEditorPause;

    Procedure FormEditorAdd;
    Procedure EmbeddedForm(Sender: TObject; aForm: TCustomForm);

    Procedure UpdateEmbedFormDsgSettings;

    property LoadedForm: TCustomForm read GetLoadedForm;

    property  OnToggleFormOrSource:TNotifyEvent read FOnToggleFormOrSource write FOnToggleFormOrSource;
  end;

//public Funtions by saulo j. Benvenutti
  function GetSysColor : Tcolor;
  function GetColorText: Tcolor;

  var FSysColor : Tcolor;
  // ------------------------------------

implementation

{$R *.frm}

var
  EMBEDDED_FORM_DESIGNER: PLazLoggerLogGroup;

function GetSysColor: Tcolor;
Var   Reg : TRegistry;
      Cint : Cardinal;
      NovaCor : array[0..3] of Byte;
begin
  {$IFDEF WINDOWS}
     try  try
     Reg := TRegistry.Create(KEY_READ);
     Reg.RootKey:= HKEY_CURRENT_USER;
     if Reg.KeyExists('SOFTWARE\Microsoft\Windows\DWM') then
        Reg.OpenKey('SOFTWARE\Microsoft\Windows\DWM', false);
        Cint := Reg.ReadInteger('ColorizationColor');
        Move(Cint, NovaCor, SizeOf(NovaCor));
        Result := RGBToColor(NovaCor[2],NovaCor[1],NovaCor[0]);
    except
      Result := clHighlight;
    end;
    finally
      FreeAndNil(Reg);
    end;
  {$ELSE}
    Result := clActiveCaption;
  {$ENDIF}

end;

function GetColorText: Tcolor;
var sum: Integer;
   begin
     try
        sum := Red(FSysColor) + Green(FSysColor) + Blue(FSysColor);
        if sum < 3*128 then     // AColor is rather "dark" ---> return white
          Result := clWhite
        else                    // AColor is rather "bright" --> return black
          Result := clBlack;

     Except
       Result := clWhite;
     end;

end;

// ==============================================================================
// =========================== TEmbedFormEditor ===================================
// ==============================================================================

procedure TEmbedFormEditor.ActivatedDesigner;
begin
  {$IFNDEF LCLQT5}
  if CT_EMBEDDED_UPDATING then exit;
  {$ENDIF}

  if FDesignPanel=nil then Exit;
  if FDesignPanel.LoadedFormIsValid=false then Exit;
  if FDesignPanel.FLoadedForm=nil then exit;

//------------------------------------------
//Fix FDesignPanel Geometry
  if FDesignPanel.FLoadedForm is TForm then
  begin
    if FDesignPanel.FIniOK=false then
     begin
       FDesignPanel.Visible:=true;
       self.Visible:=true;

       FDesignPanel.FLoadedForm.SetBounds(FDesignPanel.FLoadedForm.Left,
                                          FDesignPanel.FLoadedForm.Top,
                                          FDesignPanel.FLoadedForm.Width+1,
                                          FDesignPanel.FLoadedForm.Height);
       FDesignPanel.FLoadedForm.SetBounds(FDesignPanel.FLoadedForm.Left,
                                          FDesignPanel.FLoadedForm.Top,
                                          FDesignPanel.FLoadedForm.Width-1,
                                          FDesignPanel.FLoadedForm.Height);

       FDesignPanel.FIniOK:=true;
 
       LCLIntf.ShowWindow(FDesignPanel.FLoadedForm.Handle,SW_SHOWNORMAL);
     end;
  end;

//--------------------------------------
// Activate Embedded Form Designer
// Call main.pp procedure TMainIDE.OnDesignerActivated(Sender: TObject);


  if FDesignPanel.FLoadedForm.Designer<>nil then
    TComponentEditorDesigner(FDesignPanel.FLoadedForm.Designer).IsActive:=true;

end;



procedure TEmbedFormEditor.SetupPreviewFormDesign(_Form: TCustomForm);
begin

  if (FPreviewMoving) then  Exit;

  with pnWindowScreenPreview do
  begin
    Width  := PixelPerc(_Form.Width , CN_PREVIEW_FACTOR);
    Height := PixelPerc(_Form.Height, CN_PREVIEW_FACTOR);
    Top    := PixelPerc(_Form.Top   , CN_PREVIEW_FACTOR);
    Left   := PixelPerc(_Form.Left  , CN_PREVIEW_FACTOR);
  end;

  with pnCaptionWindowScreenPreview do
  begin
    Height := PixelPerc(Parent.Height, 15);
  end;

end;

procedure TEmbedFormEditor.DesignPanel_OnLoadForm(Sender: TObject);
begin
  //-----------------------
  {$IFDEF LCLQT5}
   if FDesignPanel.LoadedFormIsValid then
  begin
    sb_upClick(sb_left);
    sb_upClick(sb_zero);
    if Self.FDesignPanel <> nil then
    Self.FDesignPanel.PaintGuides(Self.FDesignPanel);
  end;
  {$ENDIF}

end;

procedure TEmbedFormEditor.DesignPanel_OnLoadedFormChangeBounds(Sender: TObject);
begin

  if FDesignPanel.LoadedFormIsValid then
  begin
    SetupPreviewFormDesign(FDesignPanel.FLoadedForm);
  end;

end;

function TEmbedFormEditor.PixelPerc(vl, perc: single): integer;
begin
  Result := Round((vl / 100) * perc);
end;


//------------------------ imgWindowScreenPreview Mouse Events ----------------------
procedure TEmbedFormEditor.imgWindowScreenPreviewMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FDesignPanel=nil then Exit;
  if FDesignPanel.LoadedFormIsValid=false then Exit;

  SetCapture(pnWindowScreenPreview.Handle);
  FPreviewMoving := True;
  FPreviewMovingPoint.X := x;
  FPreviewMovingPoint.Y := Y;

  pnInfoPanel.Visible:=True;
end;

procedure TEmbedFormEditor.imgWindowScreenPreviewMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
 Var R:TRect;
begin
  if FPreviewMoving=false then exit;

  pnWindowScreenPreview.SetBounds(pnWindowScreenPreview.Left - (FPreviewMovingPoint.x - X),
                                  pnWindowScreenPreview.Top  - (FPreviewMovingPoint.Y - Y),
                                  pnWindowScreenPreview.Width,
                                  pnWindowScreenPreview.Height  );

  //............................................................
  R:=Rect(round(((max(pnWindowScreenPreview.Left - (FPreviewMovingPoint.X - X), 0)) / pnWindowScreenPreview.Parent.Width)  * Screen.Width ),
          round(((max(pnWindowScreenPreview.Top  - (FPreviewMovingPoint.Y - Y), 0)) / pnWindowScreenPreview.Parent.Height) * Screen.Height),
          FDesignPanel.FLoadedForm.Width,
          FDesignPanel.FLoadedForm.Height );

  LabelX.Caption:='X: '+IntToStr(R.Left);
  LabelY.Caption:='Y: '+IntToStr(R.Top);
  LabelW.Caption:='W: '+IntToStr(R.Right);
  LabelH.Caption:='H: '+IntToStr(R.Bottom);
end;

procedure TEmbedFormEditor.imgWindowScreenPreviewMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FPreviewMoving then
  begin
    ReleaseCapture;
    FPreviewMoving := False;
    pnInfoPanel.Visible:=False;

    with pnWindowScreenPreview do
    begin
      Left := max(Left - (FPreviewMovingPoint.X - X), 0);
      Top :=  max(Top -  (FPreviewMovingPoint.Y - Y), 0);

      if FDesignPanel.LoadedForm<>nil then
      begin
        FDesignPanel.FLoadedForm.SetBounds(round((Left / Parent.Width)  * Screen.Width ),
                                           round((Top  / Parent.Height) * Screen.Height),
                                           FDesignPanel.FLoadedForm.Width,
                                           FDesignPanel.FLoadedForm.Height );

      end;
    end;
  end;
end;

//------------------------ Mouse Events ----------------------

procedure TEmbedFormEditor.sb_upClick(Sender: TObject);
begin
  if FDesignPanel=nil then Exit;
  if FDesignPanel.LoadedFormIsValid=false then Exit;

  imgWindowScreenPreviewMouseDown(nil,mbLeft, [ssLeft], 0, 0);

  if Sender = sb_up then pnWindowScreenPreview.Top  := pnWindowScreenPreview.Top  - 5;
  if Sender = sb_bottom then pnWindowScreenPreview.Top  := pnWindowScreenPreview.Top  + 5;
  if Sender = sb_left then pnWindowScreenPreview.Left := pnWindowScreenPreview.Left - 5;
  if Sender = sb_right then pnWindowScreenPreview.Left := pnWindowScreenPreview.Left + 5;
  if Sender = sb_zero then
                                begin
                                   pnWindowScreenPreview.Top  := CN_BORDER_PX;
                                   pnWindowScreenPreview.Left := CN_BORDER_PX;
                                end;

  if Sender = sb_update then  // Update button 5555
                                begin
                                   ActivatedDesigner;
                                   DesignPanel_OnLoadForm(nil);
                                   DesignPanel_OnLoadedFormChangeBounds(nil);
                                   FDesignPanel.Paint;
                                   FDesignPanel.FFakeWindowCaption.Repaint;
                                end;

  imgWindowScreenPreviewMouseUp( nil,mbLeft, [ssLeft], 0, 0);
end;

procedure TEmbedFormEditor.sb_hideClick(Sender: TObject);
begin
  if pnScreenPreview.Visible then
    begin
      pnScreenPreview.Visible:=False;
      pnScreenPreviewBtns.Visible:=False;
    end else
    begin
      pnScreenPreview.Visible:=True;
      pnScreenPreviewBtns.Visible:=True;
    end;
end;

//===========================================================================
//======================= TEmbedDesignPanel ====================================
//===========================================================================

constructor TEmbedDesignPanel.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  FLoadedForm:=nil;
  FFakeMainMenu:=nil;
  dsParentPage:=nil;
  dsParentFormEditor:=nil;
  FSysColor:=GetSysColor;
  SetInitialBounds(0,0,screen.Width,Screen.Height);

  BevelInner := bvNone;
  BevelOuter := bvNone;

  FHasLoadedForm := False;
  FIniOK:=false;

  FFakeWindow_MouseDown := false;

  FExecuteBeforeMessage := True;

  FMessagesHandler:=TEmbedMessageHandler.Create;

  FFakeWindow := TPanel.Create(Self);
  with FFakeWindow do
  begin
    BorderWidth := 0; // CN_BORDER_PX;
    BorderStyle := bsNone;
    BevelInner := bvNone;
    BevelOuter := bvRaised;
    BevelColor := FSysColor;
    OnResize    := @FFakeWindowResize;
    // retire default events
    //OnMouseDown := @FFakeWindowMouseDown;
    //OnMouseMove := @FFakeWindowMouseMove;
    //OnMouseUp   := @FFakeWindowMouseUp;
  end;

   FvertResize := TPanel.Create(Self);
  with FvertResize do
  begin
    tag := 100;
    parent := self;
    BorderStyle := bsNone;
    BorderWidth := 0;
    BevelInner := bvNone;
    BevelOuter := bvnone;
    Cursor := crSizeNS;
    OnMouseEnter:= @guidestResizeonMouseEnter;
    OnMouseLeave:= @guidesResizeMouseLeave;
    OnMouseUp   :=  @FFakeWindowMouseUp;
    OnMouseDown := @guidesResizeMouseDown;
    OnMouseMove := @guidesResizeMouseMove;
    onPaint := @guidesPaint;
    visible := true;
  end;

  FhorizontalResize := TPanel.Create(Self);
  with FhorizontalResize do
  begin
    tag := 200;
    parent := self;
    BorderStyle := bsNone;
    BorderWidth := 0;
    BevelInner := bvNone;
    BevelOuter := bvnone;
    Cursor := crSizeWE;
    OnMouseEnter:= @guidestResizeonMouseEnter;
    OnMouseLeave:= @guidesResizeMouseLeave;
    OnMouseUp   :=  @FFakeWindowMouseUp;
    OnMouseDown := @guidesResizeMouseDown;
    OnMouseMove := @guidesResizeMouseMove;
    onPaint := @guidesPaint;
    visible := true;
  end;

  FcornerVResize := TPanel.Create(Self);
  with  FcornerVResize do
  begin
    tag := 500;
    parent := self;
    BorderStyle := bsNone;
    BorderWidth := 0;
    BevelInner := bvNone;
    BevelOuter := bvnone;
    Cursor := crSizeNWSE;
    OnMouseEnter:= @guidestResizeonMouseEnter;
    OnMouseLeave:= @guidesResizeMouseLeave;
    OnMouseUp   :=  @FFakeWindowMouseUp;
    OnMouseDown := @guidesResizeMouseDown;
    OnMouseMove := @guidesResizeMouseMove;
    onPaint := @guidesPaint;
    visible := true;
    end;

  FcornerHResize := TPanel.Create(Self);
  with  FcornerHResize do
  begin
    tag := 500;
    parent := self;
    BorderStyle := bsNone;
    BorderWidth := 0;
    BevelInner := bvNone;
    BevelOuter := bvnone;
    Cursor := crSizeNWSE;
    OnMouseEnter:= @guidestResizeonMouseEnter;
    OnMouseLeave:= @guidesResizeMouseLeave;
    OnMouseUp   :=  @FFakeWindowMouseUp;
    OnMouseDown := @guidesResizeMouseDown;
    OnMouseMove := @guidesResizeMouseMove;
    onPaint := @guidesPaint;
    visible := true;
  end;



  FFakeWindowCaption := TPanel.Create(FFakeWindow);
  with FFakeWindowCaption do
  begin
    Parent := FFakeWindow;
    Align := alTop;
    Height := CN_CAPTION_HEIGHT;
    BevelInner := bvNone;
    BevelOuter := bvNone;
    ParentColor := False;
    Color := FsysColor;
    OnPaint := @FFakeWindowCaptionPaint;
  end;

  FFakeIcon := TImage.create(FFakeWindowCaption);
  with FFakeIcon do
  begin
    Parent := FFakeWindowCaption;
    BevelInner := bvNone;
    BevelOuter := bvNone;
    BorderStyle := bsNone;
    BorderWidth := 0;
    Picture.Icon := Application.icon;
    Height := Tpanel(Parent).Height;
    Width  := Height;
    Center:=True;
    top := 0;
    left := 0;
    Anchors := [AkTop,akBottom,akLeft];
    tag := 9;
    Visible:= True;
  end;

  FFakeBtnSystem := TPanel.create(FFakeWindowCaption);
  with FFakeBtnSystem do
  begin
    Parent := FFakeWindowCaption;
    BevelInner := bvNone;
    BevelOuter := bvNone;
    BorderStyle := bsNone;
    BorderWidth := 0;
    Height := CN_CAPTION_HEIGHT;
    Width  := (Height + (Height div 2));
    top := 0;
    left := (FFakeWindowCaption.width - Width);
    Anchors := [AkTop,akBottom,akRight];
    Color := Twincontrol(Parent).Color;
    OnPaint:= @FFakeButtonsPaint;
    OnMouseEnter:= @FFakeButtonsMouseEnter;
    OnMouseLeave:= @FFakeButtonsMouseLeave;
    tag := 0;
    Visible:= True;
  end;

  FFakeBtnMaximize := TPanel.create(FFakeWindowCaption);
  with FFakeBtnMaximize do
  begin
    Parent := FFakeWindowCaption;
    BevelInner := bvNone;
    BevelOuter := bvNone;
    BorderStyle := bsNone;
    BorderWidth := 0;
    Height := CN_CAPTION_HEIGHT;
    Width := (Height + (Height div 2));
    top := 0;
    left := (FFakeBtnSystem.left - Width);
    Anchors := [AkTop,akBottom,akRight];
    Caption := '';
    Color := Twincontrol(Parent).Color;
    OnPaint:= @FFakeButtonsPaint;
    OnMouseEnter:= @FFakeButtonsMouseEnter;
    OnMouseLeave:= @FFakeButtonsMouseLeave;
    tag := 2;
    Visible:= True;
  end;

  FFakeBtnMinimize := TPanel.create(FFakeWindowCaption);
  with FFakeBtnMinimize do
  begin
    Parent := FFakeWindowCaption;
    BevelInner := bvNone;
    BevelOuter := bvNone;
    BorderStyle := bsNone;
    BorderWidth := 0;
    Height := CN_CAPTION_HEIGHT;
    Width := (Height + (Height div 2));
    top := 0;
    left := (FFakeBtnMaximize.left - Width);
    Anchors := [AkTop,akBottom,akRight];
    Caption := '';
    Color := Twincontrol(Parent).Color;
    OnPaint:= @FFakeButtonsPaint;
    OnMouseEnter:= @FFakeButtonsMouseEnter;
    OnMouseLeave:= @FFakeButtonsMouseLeave;
    tag := 4;
    Visible:= true;
  end;
  FFakeWindow_MouseDown := False;
  self.OnPaint:= @PaintGuides;


end;

destructor TEmbedDesignPanel.Destroy;
begin
  if FormIsValid(FLoadedForm) then UnLoadLoadedForm;
  FMessagesHandler.Free;
  inherited Destroy;
end;

procedure TEmbedDesignPanel.EmbeddedForm(Sender: TObject; aForm: TCustomForm);
begin
  if CT_USE_EMBEDDED_DESIGNER=FALSE then Exit;

  if Embed_IsDesignForm(aForm) then AddForm(aForm);
end;

procedure TEmbedDesignPanel.AddForm(aForm: TCustomForm);
var ncaption : Pchar;
    ncaptionStr : String;
begin

  if aForm=nil then Exit;
  if FormIsValid(FLoadedForm) then Exit;

  if (aForm.ParentWindow = Self.Handle) or (aForm.Parent = Self) then
   begin
     ShowMessage('[DEV IDE ERROR 201 ] Try to EMBEDDED Again Form: '+aForm.Name);
     Exit;
   end;

 
  {$IFDEF LCLQT5}
    FMessagesHandler.UnHookControl;
    Application.ProcessMessages;
  {$ENDIF}

  FLoadedForm:=aForm;
  if (FLoadedForm is Tform) then
   begin
     FoldBorder:= FLoadedForm.BorderStyle;
     FoldBIMaximize:= biMaximize  in FloadedForm.BorderIcons;
     FoldBIMinimize:= biMinimize  in FloadedForm.BorderIcons;
     FoldBISysmenu:= biSystemMenu in FloadedForm.BorderIcons;
   end else
   begin
     FoldBorder:= bsNone;
     FoldBIMaximize:= False;
     FoldBIMinimize:= False;
     FoldBISysmenu:=  False;
   end;

  HideWindowDesign(FLoadedForm);

//------------------------------

 if dsParentPage<>nil then dsParentPage.FormEditorAdd;
{$IFNDEF LCLQT5}
 FMessagesHandler.UnHookControl;
{$ENDIF}
 FMessagesHandler.HookControl(FLoadedForm);
 FMessagesHandler.OnBeforeMessage := @Self.Form_BeforeMessage;
 FMessagesHandler.OnAfterMessage  := @Self.Form_AfterMessage;

 FHasLoadedForm := True;
 {$IFDEF LCLQT5}
 self.Invalidate;
 {$ENDIF}
end;

procedure TEmbedDesignPanel.UnLoadLoadedForm;
begin
  if LoadedFormIsValid=False then Exit;
  try
    if FFakeWindow<>nil then FFakeWindow.Visible := false;
    if FMessagesHandler.HasHookControl then FMessagesHandler.UnHookControl;
    FLoadedForm:=nil;
  finally
    FHasLoadedForm := False;
  end;
end;

procedure TEmbedDesignPanel.FFakeButtonsMouseEnter(Sender: Tobject);
function GetHighlightColor(BaseColor: TColor): TColor;
     begin
       Result := RGB(Min(GetRValue(ColorToRGB(BaseColor)) + 16, 255),
       Min(GetGValue(ColorToRGB(BaseColor)) + 16, 255),
       Min(GetBValue(ColorToRGB(BaseColor)) + 16, 255));
     end;
begin
    // teste
    if TCustomPanel(sender).tag = 0 then
    begin
       TCustomPanel(sender).Color := RGB(210,0,0);
    end
    else
       TCustomPanel(sender).Color := GetHighlightColor(TCustomPanel(sender).Color);
end;

procedure TEmbedDesignPanel.FFakeButtonsMouseLeave(Sender: Tobject);
begin
   if FLoadedForm = nil then Exit;
   if not (FLoadedForm is Tform) then Exit;;
   if (FLoadedForm.BorderStyle <> bsSizeToolWin) and (FLoadedForm.BorderStyle <> bsToolWindow) then
  begin
     TCustomPanel(Sender).color := FSysColor;
  end
  else
  begin
     Tpanel(sender).Color := RGB(230,86,81);
  end;
end;

procedure TEmbedDesignPanel.FFakeButtonsPaint(Sender: TObject);
 var bmp : Tbitmap;
     multiply : Integer;
 begin
   if LoadedFormIsValid=false then Exit;
   if (dsParentFormEditor <> NIL) and (FLoadedForm <> nil) and (dsParentFormEditor.ImgSystem <> Nil) then
   begin
      if not (FLoadedForm is TCustomform) then Exit;
      try
        if GetColorText = clwhite then
        multiply := 1 else
        multiply := 0;

        bmp := Tbitmap.create;
        if Tpanel(Sender).tag <> 9 then
        begin
          if (FLoadedForm.BorderStyle <> bsToolWindow) and (FLoadedForm.BorderStyle <> bsSizeToolWin) and (FLoadedForm.BorderStyle <> bsNone) then
          begin

            dsParentFormEditor.ImgSystem.GetBitmap(Tpanel(Sender).tag + multiply,bmp);

          end else
          begin
            dsParentFormEditor.ImgSystem.GetBitmap(6,bmp);
          end;
        end;

      if (FLoadedForm.BorderStyle <> bsNone) then
        Tpanel(Sender).canvas.Draw((Tpanel(Sender).canvas.Width - bmp.Width) div 2, (Tpanel(Sender).canvas.Height - bmp.Height) div 2 ,bmp);
       finally
        freeandnil(bmp);
      end;
   end;

end;

procedure TEmbedDesignPanel.FFakeButtonsDraw;
begin
if LoadedFormIsValid=false then Exit;

  if (dsParentFormEditor <> NIL) and (FLoadedForm <> nil) then
   begin

     if  not (LoadedForm is Tcustomform) then Exit;
     if FFakeWindowCaption = nil then exit;
     if FFakeBtnSystem = nil then exit;
     if FFakeBtnMaximize = nil then exit;
     if FFakeBtnMinimize = nil then exit;

     if not (biSystemMenu in FloadedForm.BorderIcons) then
      begin
         FFakeBtnSystem.Visible := False;
         FFakeBtnMaximize.Visible := False;
         FFakeBtnMinimize.Visible := False;
      end else
      begin
        FFakeBtnSystem.Visible := True;
        if (FLoadedForm.BorderStyle = bsSizeable) or (FLoadedForm.BorderStyle = bsSingle) then
         begin
         if (biMaximize in FloadedForm.BorderIcons) then
           begin
             if not (biMinimize in FloadedForm.BorderIcons) then
              begin
               FFakeBtnMaximize.Enabled := true;
               FFakeBtnMinimize.Enabled := False;
               FFakeBtnMaximize.Visible := true;
               FFakeBtnMinimize.Visible := true;
              end
             else
              begin
                FFakeBtnMaximize.Visible := true;
                FFakeBtnMinimize.Visible := true;
                FFakeBtnMaximize.Enabled := True;
                FFakeBtnMinimize.Enabled := True;
              end;
           end else
           begin
              if not (biMinimize in FloadedForm.BorderIcons) then
              begin
                FFakeBtnMaximize.Visible := False;
                FFakeBtnMinimize.Visible := False;
                FFakeBtnMaximize.Enabled := True;
                FFakeBtnMinimize.Enabled := True;
              end
              else
              begin
                 FFakeBtnMaximize.Visible := true;
                 FFakeBtnMinimize.Visible := true;
                 FFakeBtnMaximize.Enabled := False;
                 FFakeBtnMinimize.Enabled := true;
              end;
           end;
          if (biMinimize in FloadedForm.BorderIcons) then
           begin
             if not (biMaximize in FloadedForm.BorderIcons) then
              begin
               FFakeBtnMinimize.Enabled := true;
               FFakeBtnMaximize.Enabled := false;
               FFakeBtnMaximize.Visible := true;
               FFakeBtnMinimize.Visible := true;
              end
             else
              begin
                FFakeBtnMaximize.Visible := true;
                FFakeBtnMinimize.Visible := true;
                FFakeBtnMaximize.Enabled := True;
                FFakeBtnMinimize.Enabled := True;
              end;
           end else
           begin
              if not (biMaximize in FloadedForm.BorderIcons) then
              begin
                FFakeBtnMaximize.Visible := False;
                FFakeBtnMinimize.Visible := False;
                FFakeBtnMaximize.Enabled := True;
                FFakeBtnMinimize.Enabled := True;
              end
              else
              begin
                 FFakeBtnMaximize.Visible := true;
                 FFakeBtnMinimize.Visible := true;
                 FFakeBtnMinimize.Enabled := False;
                 FFakeBtnMaximize.Enabled := True;
              end;
           end;
           if (biMaximize in FloadedForm.BorderIcons) and  (biMinimize in FloadedForm.BorderIcons) then
              begin
                FFakeBtnMaximize.Visible := True;
                FFakeBtnMinimize.Visible := True;
                FFakeBtnMaximize.Enabled := True;
                FFakeBtnMinimize.Enabled := True;
              end;
         end;

        end;
    end;
end;

procedure TEmbedDesignPanel.guidestResizeonMouseEnter(Sender: TObject);
begin
   FdestaqueTag := Tpanel(Sender).tag;
   Fdestaque := True;
   if FvertResize = nil then Exit;
   if FhorizontalResize = nil then Exit;
   if FcornerVResize = nil then Exit;
   if FcornerHResize = nil then Exit;
   FvertResize.Invalidate;
   FhorizontalResize.Invalidate;
   FcornerVResize.Invalidate;

end;

procedure TEmbedDesignPanel.guidesResizeMouseLeave(Sender: TObject);
begin
   Fdestaque := False;
   FdestaqueTag := 0;
   if FvertResize = nil then Exit;
   if FhorizontalResize = nil then Exit;
   if FcornerVResize = nil then Exit;
   if FcornerHResize = nil then Exit;
   FvertResize.invalidate;
   FhorizontalResize.invalidate;
   FCornerVResize.invalidate;

end;

procedure TEmbedDesignPanel.guidesResizeMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    if LoadedFormIsValid=false then Exit;
    if FvertResize = nil then Exit;
    if FhorizontalResize = nil then Exit;
    if FcornerVResize = nil then Exit;
    if FcornerHResize = nil then Exit;

   if  Tpanel(Sender).tag = 100 then //vertical
     Begin
        FFakeWindow_MouseDown := True;
        FMouseArea:=3;
    end;
    if  Tpanel(Sender).tag = 200 then //horizontal
     Begin
        FFakeWindow_MouseDown := True;
        FMouseArea:=2;
    end;
    if  (Tpanel(Sender).tag = 500) then //corners
    Begin
        FFakeWindow_MouseDown := True;
        FMouseArea:=1;
    end;
       GetCursorPos(FFakeWindow_MousePos);

       FvertResize.Invalidate;
       FhorizontalResize.Invalidate;
       FcornerVResize.Invalidate;


      // self.Invalidate;
       dsParentFormEditor.pnInfoPanel.Visible:=True;
       dsParentFormEditor.LabelX.Caption:='X: '+IntToStr(FLoadedForm.Left);
       dsParentFormEditor.LabelY.Caption:='Y: '+IntToStr(FLoadedForm.Top);
       dsParentFormEditor.LabelW.Caption:='W: '+IntToStr(FLoadedForm.Width);
       dsParentFormEditor.LabelH.Caption:='H: '+IntToStr(FLoadedForm.Height);
end;

procedure TEmbedDesignPanel.guidesResizeMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
const
  minWidth = 120;
  minHeight = 50;
var
  newPos, delta: TPoint;
  frmPoint : TPoint;
  w, h: Integer;
begin
      if LoadedFormIsValid=false then Exit;
      if dsParentFormEditor = nil then Exit;
       if FFakeWindow_MouseDown=true then
       begin
         //{$IFNDEF LCLWIN32}
         FvertResize.Invalidate;
         FhorizontalResize.Invalidate;
         FcornerVResize.Invalidate;
        //{$ENDIF}

        GetCursorPos(newPos);
        delta.x := newpos.x - FFakeWindow_MousePos.X;
        delta.y := newpos.y - FFakeWindow_MousePos.y;
        GetCursorPos(FFakeWindow_MousePos);
        frmPoint.x := FLoadedForm.Width + delta.x;
        frmPoint.y := FLoadedForm.Height + delta.y;

      if frmPoint.X<minWidth  then frmPoint.X:=minWidth;
      if frmPoint.Y<minHeight then frmPoint.Y:=minHeight;


       dsParentFormEditor.LabelX.Caption:='X: '+IntToStr(FLoadedForm.Left);
       dsParentFormEditor.LabelY.Caption:='Y: '+IntToStr(FLoadedForm.Top);
       dsParentFormEditor.LabelW.Caption:='W: '+IntToStr(frmPoint.X);
       dsParentFormEditor.LabelH.Caption:='H: '+IntToStr(frmPoint.Y);


       if LoadedForm<>nil then     // Resize Only if Loaded Form is Valid
       begin
          case FMouseArea of
             1: begin
                 FLoadedForm.SetBounds(FLoadedForm.Left, FLoadedForm.Top, frmPoint.X, frmPoint.Y);
                end;
             2: begin
                 FLoadedForm.SetBounds(FLoadedForm.Left, FLoadedForm.Top, frmPoint.X, FLoadedForm.Height);
                end;
             3: begin
                 FLoadedForm.SetBounds(FLoadedForm.Left, FLoadedForm.Top, FLoadedForm.Width, frmPoint.Y);
                end;
          end;
          //{$IFDEF LCLQT5}
          // Self.PaintGuides(Self);
          //
          //{$ENDIF}
           Self.invalidate;
       end;

       end;


end;

procedure TEmbedDesignPanel.guidesPaint(sender: Tobject);
function GetHighlightColor(BaseColor: TColor): TColor;
      begin
        Result := RGB(Min(GetRValue(ColorToRGB(BaseColor)) + 64, 255),
        Min(GetGValue(ColorToRGB(BaseColor)) + 64, 255),
        Min(GetBValue(ColorToRGB(BaseColor)) + 64, 255));
      end;

      function GetDarkColor(BaseColor: TColor): TColor;
      begin
        Result := RGB(Max(GetRValue(ColorToRGB(BaseColor)) - 64, 0),
        Max(GetGValue(ColorToRGB(BaseColor)) - 64, 0),
        Max(GetBValue(ColorToRGB(BaseColor)) - 64,0));
      end;
 var destaquecolor : Tcolor;
     colorBase : Tcolor;
begin

 if FFakeWindow_MouseDown then
 begin
   if TPanel(sender).tag <> 500 then
   begin
    TPanel(sender).canvas.pen.Color := TPanel(sender).color;
    TPanel(sender).canvas.Brush.Color :=  TPanel(sender).color;
    TPanel(sender).canvas.rectangle(0,0,TPanel(sender).Width, TPanel(sender).Height);
   end else
   begin
    if FcornerHResize <> nil then
    with FcornerHResize.canvas do
    begin
       pen.Color := FcornerHResize.color;
       Brush.Color :=  FcornerHResize.color;
       rectangle(0,0,FcornerHResize.Width, FcornerHResize.Height);
    end;
    if FcornerVResize <> nil then
    with FcornerVResize.canvas do
    begin
       pen.Color := FcornerVResize.color;
       Brush.Color :=  FcornerVResize.color;
       rectangle(0,0,FcornerVResize.Width, FcornerVResize.Height);
    end;
   end;
   exit;
 end;
 if GetColorText = clWhite then
   begin
   destaquecolor := GetDarkColor(FSysColor);
   colorBase :=  GetHighlightColor(FSysColor);
   end else
   begin
   destaquecolor := GetHighlightColor(FSysColor);
   colorbase := GetDarkColor(FSysColor);
   end;
 if TPanel(sender).tag = 100 then //vertguide
 begin
    with TPanel(sender).canvas do
    begin
      if (Fdestaque) and (Fdestaquetag = 100) then
      begin
        Pen.Color := destaquecolor;
        Brush.Color := destaquecolor;
       end else
      begin
         Pen.Color := colorBase;
         Brush.Color := colorBase;
      end;

      Rectangle(0,(TPanel(sender).Height div 2) - 1,(FFakeWindow.Width div 2) - 8, (TPanel(sender).Height div 2) + 1);
      Ellipse((FFakeWindow.Width div 2) - 6, 0,(FFakeWindow.Width div 2) + 4, TPanel(sender).Height);
      Rectangle((FFakeWindow.Width div 2) + 6, (TPanel(sender).Height div 2) - 1,  TPanel(sender).Width, (TPanel(sender).Height div 2) + 1);

    end;
 end;
  if TPanel(sender).tag = 200 then //vertguide
 begin
    with TPanel(sender).canvas do
    begin
      if (Fdestaque) and (Fdestaquetag = 200) then
      begin
        Pen.Color := destaquecolor;
        Brush.Color := destaquecolor;
       end else
      begin
         Pen.Color := colorBase;
         Brush.Color := colorBase;
      end;
      rectangle((TPanel(sender).Width div 2) - 1,  0,  (TPanel(sender).Width div 2) + 1, (FFakeWindow.Height div 2) - 8);
      Ellipse(0,  (FFakeWindow.Height div 2) - 6,  TPanel(sender).Width, (FFakeWindow.Height div 2) + 4);
      rectangle((TPanel(sender).Width div 2) - 1, (FFakeWindow.Height div 2) + 6,  (TPanel(sender).Width div 2) + 1, TPanel(sender).Height);
    end;
 end;

 if (TPanel(sender).tag = 500) then //cornerguides
 begin
    if FcornerHResize <> nil then
    with FcornerHResize.canvas do
    begin
      if (Fdestaque) and (Fdestaquetag = 500) then
      begin
        Pen.Color := destaquecolor;
        Brush.Color := destaquecolor;
       end else
      begin
         Pen.Color := colorBase;
         Brush.Color := colorBase;
      end;
      rectangle((FcornerHResize.Width div 2) - 1,  2,  (FcornerHResize.Width div 2) + 1, FcornerHResize.Height - 2);
    end;
    if FcornerVResize <> nil then
    with FcornerVResize.canvas do
    begin
      if (Fdestaque) and (Fdestaquetag = 500) then
      begin
        Pen.Color := destaquecolor;
        Brush.Color := destaquecolor;
       end else
      begin
         Pen.Color := colorBase;
         Brush.Color := colorBase;
      end;
      rectangle(2,(FcornerVResize.Height div 2) - 1,FcornerVResize.Width  - 12, (FcornerVResize.Height div 2) + 1);
      Ellipse(FcornerVResize.Width - 10 , FcornerVResize.Height - 10 ,FcornerVResize.Width, FcornerVResize.Height);
    end;
 end;
end;

procedure TEmbedDesignPanel.PaintGuides(Sender: TObject);
begin
   if FLoadedForm = nil then exit;
   if FFakeWindow = nil then Exit;
   if not FFakeWindow.Visible then Exit;

    FFakeWindow.BevelColor:= FsysColor;

    IF FvertResize <> nil then
   begin
      FvertResize.SetBounds(FFakeWindow.left + 1 , (FFakeWindow.top + FFakeWindow.Height) + 5 , FFakeWindow.Width - 17,10);
   end;
   IF FhorizontalResize <> nil then
   begin
   FhorizontalResize.SetBounds( FFakeWindow.left + FFakeWindow.width + 5,FFakeWindow.top + 1, 10, FFakeWindow.Height - 17 );
   end;

    IF FCornerVResize <> nil then
    begin
     FCornerVResize.SetBounds(FvertResize.Left + FvertResize.Width + 2 , FvertResize.top , 30,10);
    {$IFNDEF LCLWIN32}
     FCornerVResize.SetBounds((FFakeWindow.left + FFakeWindow.Width) - 15 , (FFakeWindow.top + FFakeWindow.Height) + 5 , 30,10);
    {$ENDIF}
    end;

    IF FCornerHResize <> nil then
    begin
      FCornerHResize.SetBounds( FhorizontalResize.left, FhorizontalResize.top + FhorizontalResize.Height + 2, 10, 20);
    {$IFNDEF LCLWIN32}
      FCornerHResize.SetBounds((FFakeWindow.left + FFakeWindow.width) + 5, (FFakeWindow.top + FFakeWindow.Height) - 15, 10, 20);
    {$ENDIF}
    end;


end;


function TEmbedDesignPanel.FormIsValid(aForm: TCustomForm): Boolean;
begin
  Result := (Assigned(aForm)) and (aForm <> Nil);
end;

function TEmbedDesignPanel.LoadedFormIsValid: Boolean;
begin
  Result := FormIsValid(FLoadedForm);
end;

procedure TEmbedDesignPanel.HideWindowDesign(aForm: TCustomForm);
begin
  if (aForm = Nil) or (not Assigned(aForm)) then Exit;

//--- ct5555 --------------------
// Solve Form.Caption Problem on Cocoa and GTK3
{$IF DEFINED(LCLCocoa) or DEFINED(LCLGtk3)}
  aform.Visible:=false;
{$ELSE}
  ShowWindow(aForm.Handle, SW_HIDE);
{$ENDIF}
end;

function TEmbedDesignPanel.FormZeroOrigin(aForm: TCustomForm): TPoint;

//------------------------------------------
  procedure _FindFormBorder;
    var
        w: TWinControl;
        rc1, rc2: TRect;
   begin
    if CN_REMOVE_FORM_BORDER=False then exit;

    try
      w := TWinControl.Create(Self);
      w.Left := 0;
      w.Top := 0;
      w.Parent := aForm;

      GetWindowRect(w.Handle, rc1);
      GetWindowRect(aForm.Handle, rc2);

      Result.x := Max(rc1.Left - rc2.Left, 0);
      Result.y := Max(rc1.Top  - rc2.Top,  0);
    finally
      w.free;
    end;
   end;

//------------------------------------------
  procedure _FindCompSizes;
    var i: Integer;
   begin
     if CN_MUST_FIND_MAINMENU_HEIGHT=False then Exit;

     for i := 0 to aForm.ComponentCount - 1 do
      begin
        if aForm.Components[i] is TMainMenu then
         if TMainMenu(aForm.Components[i]).Items.Count>0 then // 8888  Don't add Height of TmainMenu with No Items
          begin
            Result.y := Result.y - LCLIntf.GetSystemMetrics(SM_CYMENU);
            Exit;
          end;
      end;
   end;

//-----------------------------------------
begin
Result := Point(0,0);

if FormIsValid(aForm)=false then exit;

if FFakeWindow_MouseDown then
begin
 result:=FZeroOriginPoint;
 EXIT;
end;

_FindFormBorder;
_FindCompSizes;

FZeroOriginPoint:=Result;
end;

function TEmbedDesignPanel.CheckForFakeMainMenu(aForm: TCustomForm): Boolean;
    var i: Integer;
        xMainMenu:TMainMenu;
begin

Result:=False;

if FormIsValid(aForm)=False then Exit;
if Embed_IsDesignForm(aform)=false then Exit;
if CN_USE_FAKE_MAINMENU=False then Exit;
if FFakeWindow_MouseDown then EXIT;

xMainMenu:=nil;

//..... Find Form MainMenu ...
for i := 0 to aForm.ComponentCount - 1 do
  begin
    if aForm.Components[i] is TMainMenu then
      if TMainMenu(aForm.Components[i]).Items.Count>0 then
       begin
        xMainMenu:=TMainMenu(aForm.Components[i]);
       end;
  end;

Result:=(xMainMenu<>nil);

//............................
if xMainMenu=nil then
begin
  if FFakeMainMenu<>nil then
    begin
      FFakeMainMenu.Free;
      FFakeWindowCaption.Height:=CN_CAPTION_HEIGHT;
    end;
    FFakeMainMenu:=nil;
end else
begin
  if FFakeWindow=nil then Exit;

  if FFakeMainMenu=nil then
  begin
     FFakeMainMenu:=TTEmbedFakeMainMenu.Create(FFakeWindowCaption);
     FFakeMainMenu.Parent:=FFakeWindowCaption;
     FFakeMainMenu.MainMenuAssign(xMainMenu);
     FFakeWindowCaption.Height:=CN_CAPTION_HEIGHT+FFakeMainMenu.Height;
  end;
end;
end;

procedure TEmbedDesignPanel.UpdateFakeMainMenu;
begin
if CN_USE_FAKE_MAINMENU=False then Exit;
if FFakeMainMenu=nil then Exit;

FFakeMainMenu.MainMenuUpdate;
end;

//=============================== Before LoadedForm Message =====================
procedure TEmbedDesignPanel.Form_BeforeMessage(Sender: TObject; SenderForm: TControl; Msg: TLMessage);
var
  pRect: TRect;
//..................................................................
  procedure _AdjustSelfPosition(aForm: TCustomForm);
  begin

    if aForm.ParentWindow <> Self.Handle then
    begin
      Self.SetBounds(0,0,Self.Width,Self.Height);
    end;

    GetWindowRect(Self.Handle, pRect);

    Self.SetBounds(-(pRect.Left - Self.Left),
                   -(pRect.Top  - Self.Top),
                     Self.Width,
                     Self.Height  );
  end;
//..................................................................
begin
if NOT FExecuteBeforeMessage     then Exit;
if NOT (Sender=FMessagesHandler) then Exit;
if LoadedFormIsValid=false       then Exit;
if TCustomForm(SenderForm)<>FLoadedForm then Exit;

//debugln('TEmbedDesignPanel.Form_BeforeMessage : '+GetMessageName(Msg.msg)); //--- ct7777 -----

//--------------------------------------------------------

           if NOT (csDestroying in FLoadedForm.ComponentState) then
            if ((CN_FORM_DESIGNER_PARENT_TYPE=ctptParentWindow) and (FLoadedForm.ParentWindow <> Self.Handle)) or
                 ((CN_FORM_DESIGNER_PARENT_TYPE=ctptParent) and (FLoadedForm.Parent <> Self)) or
                 ( CN_FORM_DESIGNER_PARENT_TYPE=ctptDock) or
                 ( CN_FORM_DESIGNER_PARENT_TYPE=ctptNone) then
               begin
                   try
                         FExecuteBeforeMessage := False;
                         FLoadedForm.BeginUpdateBounds;
                         pRect.Left   := FLoadedForm.Left;
                         pRect.Top    := FLoadedForm.Top;
                         pRect.Right  := FLoadedForm.Left + FLoadedForm.Width;
                         pRect.Bottom := FLoadedForm.Top  + FLoadedForm.Height;

                         case CN_FORM_DESIGNER_PARENT_TYPE of
                           ctptParentWindow  : FLoadedForm.ParentWindow := Self.Handle;
                           ctptParent        : FLoadedForm.Parent := Self;
                           ctptDock          : FLoadedForm.Dock(Self, pRect);
                           ctptNone          : ;
                         end;


                    finally
                           FLoadedForm.EndUpdateBounds;
                            // saulo
                          // _AdjustSelfPosition(FLoadedForm);
                           FExecuteBeforeMessage := True;


                    end;

               if Assigned(OnFormLoad) then OnFormLoad(Self);
               if Assigned(OnLoadedFormChangeBounds) then  OnLoadedFormChangeBounds(Self);
              end;

 //--------------------------------------------------------
end;

//=============================== After LoadedForm Message =====================
procedure TEmbedDesignPanel.Form_AfterMessage(Sender: TObject; SenderForm: TControl; Msg: TLMessage);
var pRect: TRect;

//..................................................................
  procedure _UpdateFakeWindow(aForm: TCustomForm);
   var zoPoint:TPoint;
  begin
      FFakeWindow.Visible := ((aForm <> Nil) and (Assigned(aForm)));

      if FFakeWindow.Visible then
      begin
        zoPoint:=FormZeroOrigin(aForm);
        FFakeWindow.SetBounds(
                              ((aForm.Left)  - (FFakeWindowCaption.Left)) + zoPoint.x,
                              ((aForm.Top )  - (FFakeWindowCaption.Top + FFakeWindowCaption.Height)) + zoPoint.y,
                                aForm.Width  + (FFakeWindowCaption.Left * 2),
                                aForm.Height + (FFakeWindowCaption.Top + FFakeWindowCaption.Height) + FFakeWindowCaption.Top
                              );
      end;

      if FFakeWindow_MouseDown then exit;
      FFakeWindowResize(FFakeWindow);
  end;
//..................................................................
  procedure _UpdateLoadedFormBorder(aForm: TCustomForm);
  var
    xRegion: TRegion;
  begin
    if CN_REMOVE_FORM_BORDER=false then Exit;

    with aForm, FormZeroOrigin(aForm) do
    begin
      xRegion := TRegion.Create;
       try
         xRegion.AddRectangle(  x,
                                y,
                                x + Width,
                                y + Height );

         aForm.SetShape(xRegion);
       finally
         xRegion.Free;
      end;

    end;
  end;
//..................................................................
  procedure _UpdateFormDesigner(aForm: TCustomForm);
  begin
    with TCustomForm(aForm) do UpdateControlState;
  end;
//..................................................................
  procedure _UpdateSelf;
    var R:TRect;
   begin

     if FFakeWindow.Visible then
         R:=Rect((-FFakeWindow.Left) + 5,(-FFakeWindow.Top)  + 5, Screen.Width, Screen.Height) else
         R:=Rect((-FLoadedForm.Left) + 5,(-FLoadedForm.Top)  + 5, Screen.Width, Screen.Height);

      Self.SetBounds(R.Left, R.Top, R.Right, R.Bottom);
      {$IFDEF LCLQT5}
      Self.PaintGuides(self);
      Self.Paint;
      {$ENDIF}
   end;
//..................................................................
begin
if FHasLoadedForm=false then exit;
if NOT (Sender=FMessagesHandler) then Exit;
if LoadedFormIsValid=false then Exit;
if TCustomForm(SenderForm)<>FLoadedForm  then Exit;

if (csDestroying in FLoadedForm.ComponentState) then Exit;
if (csFreeNotification in FLoadedForm.ComponentState) then Exit;
if (csLoading in FLoadedForm.ComponentState) then Exit;
if (csUpdating in FLoadedForm.ComponentState) then Exit;
if (csReading in FLoadedForm.ComponentState) then Exit;
if (csWriting in FLoadedForm.ComponentState) then Exit;


//--------------------------------
case Msg.msg of
LM_SHOWWINDOW:
              begin
              if TLMShowWindow(Msg).Show then
                begin

                  if CN_USE_FAKE_MAINMENU then
                    if CheckForFakeMainMenu(FLoadedForm) then
                       UpdateFakeMainMenu;

                  _UpdateFakeWindow(FLoadedForm);
                  _UpdateLoadedFormBorder(FLoadedForm);
                  _UpdateSelf;
                end;
              end;

LM_ACTIVATE,
LM_SETFOCUS,
LM_MOVE,
LM_SIZE,
CM_SHOWINGCHANGED:
             begin
                 If FSysColor <> GetSysColor then
                    Begin
                     FSysColor:= GetSysColor;
                     if FFakeWindowCaption <> nil then
                      begin
                        FFakeWindowCaption.Color:= FSysColor;
                        FFakeWindowCaption.Font.Color := GetColorText;
                        FFakeBtnMaximize.Color:= FSysColor;
                        FFakeBtnMinimize.Color:= FSysColor;
                        FFakeBtnSystem.Color:= FSysColor;
                        FFakeWindow.BevelColor:= FSysColor;
                     end;

                    end;
                 _UpdateFakeWindow(FLoadedForm);
                 _UpdateLoadedFormBorder(FLoadedForm);
                 _UpdateSelf;
                 _UpdateFormDesigner(FLoadedForm);
                 if Assigned(OnLoadedFormChangeBounds) then  OnLoadedFormChangeBounds(Self);
                 if FFakeWindowCaption<>nil then FFakeWindowCaption.Repaint;

             end;

CM_TEXTCHANGED:    //Change Stored Form/Module/Frame Caption/Name
             begin
                if FFakeWindowCaption<>nil then
                 FFakeWindowCaption.Repaint;
             end;

CM_MENUCHANGED,
CM_CONTROLLISTCHANGE:   // for Fake MainMemu added
             begin

               if CN_USE_FAKE_MAINMENU=false then
                 begin   // for Original MainMemu added

                    if dsParentFormEditor<>NIL then
                      dsParentFormEditor.sb_upClick(dsParentFormEditor.sb_update);

                  end else
                  begin   // for Fake MainMemu added

                    if FFakeMainMenu=nil then   // Used Only for Init Greate FakeMainMenu
                     if CheckForFakeMainMenu(FLoadedForm) then
                     begin
                       UpdateFakeMainMenu;
                       _UpdateFakeWindow(FLoadedForm);
                       _UpdateLoadedFormBorder(FLoadedForm);
                       _UpdateSelf;
                       if Assigned(OnLoadedFormChangeBounds) then  OnLoadedFormChangeBounds(Self);
                    end;

                 end;

             end;

LM_ERASEBKGND,
LM_PAINT:        // for Fake MainMemu Deleted
             begin
                if FLoadedForm = nil then Exit;
                if FFakeWindowCaption <> nil then
                begin
                  if not (LoadedForm is TCustomform) then
                   begin
                       FoldBorder := Bsnone;
                       FFakeWindowCaption.Height:=0;
                   end else
                   begin
                      if FSysColor <> GetSysColor then
                      begin
                        FSysColor := GetSysColor;
                        If FFakeWindowCaption <> nil then
                        FFakeWindowCaption.Color := FSysColor;
                        If FFakeWindow <> nil then
                        FFakeWindow.BevelColor:= Fsyscolor;
                      end;
                      if FLoadedForm.BorderStyle <> FoldBorder then
                       begin
                          FoldBorder := FLoadedForm.BorderStyle;
                          IF FLoadedForm.BorderStyle = BsNone then
                          begin
                             FFakeWindowCaption.Height:=0;
                          end;
                          IF FLoadedForm.BorderStyle = bsDialog then
                          begin
                                FFakeWindowCaption.Height:=CN_CAPTION_HEIGHT - 5;
                                FFakeBtnMaximize.Visible:=False;
                                FFakeBtnMinimize.Visible:=False;
                                FFakeBtnSystem.Height   := FFakeWindowCaption.Height;
                                FFakeBtnSystem.Width    := FFakeBtnMaximize.Width;
                                FFakeBtnSystem.left     := FFakeBtnSystem.Parent.Width - FFakeBtnSystem.Width;
                                FFakeBtnSystem.Color    :=  FSysColor;
                                FFakeBtnSystem.Top:= 0;
                          end;
                          IF FLoadedForm.BorderStyle = bsSingle then
                          begin
                                FFakeIcon.Visible:=True;
                                FFakeWindowCaption.Height:=CN_CAPTION_HEIGHT - 5;
                                FFakeBtnMaximize.Visible:=true;
                                FFakeBtnMaximize.Enabled:=false;
                                FFakeBtnMinimize.Visible:=true;
                                FFakeBtnSystem.Height   := FFakeWindowCaption.Height;
                                FFakeBtnSystem.Width    := FFakeBtnMaximize.Width;
                                FFakeBtnSystem.left := FFakeBtnSystem.Parent.Width - FFakeBtnSystem.Width;
                                FFakeBtnSystem.Color:=  FSysColor;
                                FFakeBtnSystem.Top:= 0;
                          end;
                          IF FLoadedForm.BorderStyle = bsToolWindow then
                          begin
                                FFakeIcon.Visible:=false;
                                FFakeWindowCaption.Height := CN_CAPTION_HEIGHT - 5;
                                FFakeBtnSystem.Height := FFakeWindowCaption.Height - 6;
                                FFakeBtnSystem.Width := FFakeBtnSystem.Height;
                                FFakeBtnSystem.left := FFakeBtnSystem.Parent.Width - FFakeBtnSystem.Width;
                                FFakeBtnSystem.Color:=  RGB(230,86,81);
                                FFakeBtnSystem.Top := 3;
                                FFakeBtnMaximize.Visible:=false;
                                FFakeBtnMinimize.Visible:=false;
                          end ;
                          IF FLoadedForm.BorderStyle = bsSizeToolWin then
                          begin
                                FFakeIcon.Visible:=False;
                                FFakeWindowCaption.Height:=CN_CAPTION_HEIGHT;
                                FFakeBtnSystem.Height:= FFakeWindowCaption.Height - 11;
                                FFakeBtnMaximize.Visible:=false;
                                FFakeBtnMinimize.Visible:=false;
                                FFakeBtnSystem.Width:= FFakeBtnSystem.Height;
                                FFakeBtnSystem.left := FFakeBtnSystem.Parent.Width - FFakeBtnSystem.Width;
                                FFakeBtnSystem.Color:=   RGB(230,86,81);
                                FFakeBtnSystem.Top :=  8;
                          end ;
                          IF FLoadedForm.BorderStyle = bsSizeable then
                          begin
                               FFakeIcon.Visible:=True;
                               FFakeWindowCaption.Height := CN_CAPTION_HEIGHT;
                               FFakeBtnMaximize.Visible:=True;
                               FFakeBtnMinimize.Visible:=True;
                               FFakeBtnMaximize.Enabled:=True;
                               FFakeBtnSystem.Height := FFakeWindowCaption.Height;
                               FFakeBtnSystem.Width  := FFakeBtnMaximize.Width;
                               FFakeBtnSystem.Top:= 0;
                               FFakeBtnSystem.Color:=  FSysColor;
                               FFakeBtnSystem.Width := (FFakeBtnSystem.Height + (FFakeBtnSystem.Height div 2));
                               FFakeBtnSystem.left := FFakeBtnSystem.Parent.Width - FFakeBtnSystem.Width;
                          end;
                            FFakeButtonsDraw;
                            if FFakeWindowCaption<>nil then
                               FFakeWindowCaption.Repaint;
                           _UpdateLoadedFormBorder(FLoadedForm);
                           _UpdateFakeWindow(FLoadedForm);
                           _UpdateSelf;
                           _UpdateFormDesigner(FLoadedForm);

                       end;

                       if ((FoldBIMaximize) <> (biMaximize in FloadedForm.BorderIcons)) or
                          ((FoldBIMinimize) <> (biMinimize in FloadedForm.BorderIcons)) or
                          ((FoldBISysmenu) <> (biSystemMenu in FloadedForm.BorderIcons))
                       then
                       begin
                           FoldBIMaximize:= biMaximize in FloadedForm.BorderIcons;
                           FoldBIMinimize:= biMinimize in FloadedForm.BorderIcons;
                           FoldBISysmenu:= biSystemMenu in FloadedForm.BorderIcons;
                           FFakeButtonsDraw;
                           _UpdateLoadedFormBorder(FLoadedForm);
                           _UpdateFakeWindow(FLoadedForm);
                           _UpdateSelf;
                           _UpdateFormDesigner(FLoadedForm);
                       end;


                      if CN_USE_FAKE_MAINMENU=false then exit;
                      if FFakeMainMenu=nil then exit;    //Used Only for Init Greate FakeMainMenu

                      if CheckForFakeMainMenu(FLoadedForm) then   //  FakeMainMenu Exists so Update Items
                       begin
                         UpdateFakeMainMenu;
                       end else                                   //  FakeMainMenu Deleted so Update DesignPanel
                       begin
                         _UpdateFakeWindow(FLoadedForm);
                         _UpdateLoadedFormBorder(FLoadedForm);
                         _UpdateSelf;
                         if Assigned(OnLoadedFormChangeBounds) then  OnLoadedFormChangeBounds(Self);
                       end;
                  end;
                end;
             end;
end;
end;

//Result=0 Mouse is in LoadedForm
//Result=1 Mouse in Bottom-Righr Corner
//Result=2 Mouse in Righr Bar
//Result=3 Mouse in Bottom Bar
function TEmbedDesignPanel.FindMouseArea(const X, Y: integer): integer;
begin
   result:=0;

   if (x < FFakeWindow.Width   - 4*CN_BORDER_PX) and
      (y >= FFakeWindow.Height - CN_BORDER_PX)
    then  begin result:=3; exit; end;   // Bottom Bar

   if (x >= FFakeWindow.Width  - CN_BORDER_PX) and
      (y <  FFakeWindow.Height - 4*CN_BORDER_PX)
    then  begin result:=2; exit; end;   // Right Bar

   if (x >= FFakeWindow.Width  - CN_BORDER_PX) or
      (y >= FFakeWindow.Height - CN_BORDER_PX)
    then begin result:=1; exit; end;    // Bottom-Righr Corner

end;

procedure TEmbedDesignPanel.FFakeWindowMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if LoadedFormIsValid=false then Exit;

  FFakeWindow_MouseDown := false;

  if
    (x >= FFakeWindow.Width  - CN_BORDER_PX) or
    (y >= FFakeWindow.Height - CN_BORDER_PX) then
  begin
    FFakeWindow_MouseDown := True;
    GetCursorPos(FFakeWindow_MousePos);

    FDeltaPointMousePos:=Point(FFakeWindowCaption.Left-X,FFakeWindowCaption.Top-Y);

    dsParentFormEditor.pnInfoPanel.Visible:=True;
  end;

end;

procedure TEmbedDesignPanel.FFakeWindowMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
const
  minWidth = 120;
  minHeight = 50;
var
  newPos: TPoint;
  frmPoint : TPoint;
  w, h: Integer;
begin

if Sender= nil then Exit;

with TWinControl(Sender) do
begin

  if FFakeWindow_MouseDown=false then  // NO Resize Mode
    begin
     FMouseArea:=FindMouseArea(x,y);
     case FMouseArea of
       1: Cursor := crSizeNWSE; // Bottom-Right Corner
       2: Cursor := crSizeWE;   // Right Bar
       3: Cursor := crSizeNS;   // Bottom Bar
     else
       Cursor := crDefault;
     end;
  end else                            // Resize Mode
  begin
      GetCursorPos(newPos);
      frmPoint := ScreenToClient(Mouse.CursorPos);

      if CN_REMOVE_FORM_BORDER then
       begin
         frmPoint.X:=frmPoint.X-FZeroOriginPoint.X;
         frmPoint.Y:=frmPoint.Y-FZeroOriginPoint.Y;
       end else
       begin
         frmPoint.X:=frmPoint.X-CN_BORDER_PX;
         frmPoint.Y:=frmPoint.Y-CN_BORDER_PX-FFakeWindowCaption.Height;
       end;

      if frmPoint.X<minWidth  then frmPoint.X:=minWidth;
      if frmPoint.Y<minHeight then frmPoint.Y:=minHeight;


       dsParentFormEditor.LabelX.Caption:='X: '+IntToStr(FLoadedForm.Left);
       dsParentFormEditor.LabelY.Caption:='Y: '+IntToStr(FLoadedForm.Top);
       dsParentFormEditor.LabelW.Caption:='W: '+IntToStr(frmPoint.X);
       dsParentFormEditor.LabelH.Caption:='H: '+IntToStr(frmPoint.Y);


       if LoadedForm<>nil then     // Resize Only if Loaded Form is Valid
       begin
          case FMouseArea of
             1: begin
                 FLoadedForm.SetBounds(FLoadedForm.Left, FLoadedForm.Top, frmPoint.X, frmPoint.Y);
                end;
             2: begin
                 FLoadedForm.SetBounds(FLoadedForm.Left, FLoadedForm.Top, frmPoint.X, FLoadedForm.Height);
                end;
             3: begin
                 FLoadedForm.SetBounds(FLoadedForm.Left, FLoadedForm.Top, FLoadedForm.Width, frmPoint.Y);
                end;
          end;
         {$IFDEF LCLQT5}
          Self.Invalidate;
         {$ENDIF}
       end;
    end;
end;
end;

procedure TEmbedDesignPanel.FFakeWindowMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FFakeWindow_MouseDown then
  begin
    FFakeWindow_MouseDown := False;
    dsParentFormEditor.pnInfoPanel.Visible:=False;
    if FvertResize = nil then Exit;
    if FhorizontalResize = nil then Exit;
    if FcornerVResize = nil then Exit;
    if FcornerHResize = nil then Exit;
     FvertResize.Invalidate;
     FhorizontalResize.Invalidate;
     FcornerVResize.Invalidate;
  end;
end;

procedure TEmbedDesignPanel.FFakeWindowResize(Sender: TObject);
var
  fk_rgn, fm_rgn, Region, Region2 : HRGN;
  fk_rc, fm_rc: TRect;
  xRegion : TRegion;
begin

  fk_rc.Left   := 0;
  fk_rc.Top    := 0;
  fk_rc.Right  := FFakeWindow.Width;
  fk_rc.Bottom := FFakeWindow.Height;

  with fm_rc do
  begin
    Left   := FFakeWindowCaption.Left;
    Top    := FFakeWindowCaption.Top + FFakeWindowCaption.Height;
    Right  := Left + FFakeWindowCaption.Width;
    Bottom := FFakeWindow.Height - (FFakeWindowCaption.Top);
  end;

  fk_rgn := CreateRectRgn(fk_rc.Left, fk_rc.Top, fk_rc.Right, fk_rc.Bottom);
  fm_rgn := CreateRectRgn(fm_rc.Left, fm_rc.Top, fm_rc.Right, fm_rc.Bottom);

  CombineRgn(fk_rgn, fk_rgn, fm_rgn, RGN_DIFF);

  //........................................
  GetRgnBox(fk_rgn,PRect(@fk_rc));

  xRegion := TRegion.Create;
       try
         xRegion.AddRectangle(
                                fk_rc.Left,
                                fk_rc.Top,
                                fk_rc.Right,
                                fk_rc.Bottom);

         FFakeWindow.SetShape(xRegion);
       finally
         xRegion.Free;
      end;

end;

procedure TEmbedDesignPanel.FFakeWindowCaptionPaint(Sender: TObject);
 Var xss : string;
    nCaption : array[0..255] of Char;
begin
  if LoadedFormIsValid=false then Exit;

  if FLoadedForm<>Nil then xss:= FLoadedForm.Caption else Exit;

  FFakeWindowCaption.Color := FsysColor;

  if (Pos('TFRAME',Uppercase(FLoadedForm.ClassName)) > 0) OR (Pos('TDATAMODULE',Uppercase(FLoadedForm.ClassName)) > 0) then
    begin
     FFakeWindowCaption.Height:=0;
     Exit;
    end;

    if Xss <> '' then
    begin
      FFakeWindowCaption.Canvas.Font.Color := GetColorText;
      if (FFakeIcon <> nil) and (FFakeIcon.Visible) then
      FFakeWindowCaption.Canvas.TextOut(FFakeIcon.Width+2, (FFakeWindowCaption.ClientHeight - FFakeWindowCaption.Canvas.Font.GetTextHeight(xss)) div 2, xss)
      else
      FFakeWindowCaption.Canvas.TextOut( 10, (FFakeWindowCaption.ClientHeight - FFakeWindowCaption.Canvas.Font.GetTextHeight(xss)) div 2, xss);
    end;
end;

procedure TEmbedDesignPanel.InitializeWnd;
begin
  inherited InitializeWnd;

  with FFakeWindow do
  begin
    Parent := Self;
    ParentColor := False;
    Color := clForm;
  end;

end;

//===========================================================================
//======================== TEmbedMessageHandler =============================
//===========================================================================

constructor TEmbedMessageHandler.Create;
begin
  inherited Create;
  SetDefaulOptions;
  FHookControl    := nil;
  OnBeforeMessage := nil;
  OnAfterMessage  := nil;
end;

destructor TEmbedMessageHandler.Destroy;
begin
  UnHookControl;
  inherited Destroy;
end;

Procedure TEmbedMessageHandler.SetDefaulOptions;
begin
 FOptions:=[ehoSendBefore,ehoSendAfter,ehoSendToForm];
end;

procedure TEmbedMessageHandler.HookControl(AControl: TControl);
begin
  SetDefaulOptions;

  if AControl=nil then exit;
  FHookControl := AControl;
  InheritedWndMethod :=  FHookControl.WindowProc; //Save Control WindowProc
  FHookControl.WindowProc := @SelfWndMethod;      //Change Control WindowProc
end;

procedure TEmbedMessageHandler.UnHookControl;
begin
  OnBeforeMessage := nil;
  OnAfterMessage  := nil;
  if NOT HasHookControl then Exit;
  FHookControl.WindowProc:=InheritedWndMethod;   //Restore Control WindowProc
  FHookControl:=nil;
  InheritedWndMethod:=nil;

  SetDefaulOptions;
end;

function TEmbedMessageHandler.HasHookControl: Boolean;
begin
  Result := (Assigned(FHookControl)) and (FHookControl <> Nil);

end;

procedure TEmbedMessageHandler.SelfWndMethod(var TheMessage: TLMessage);
var Aux : String;
begin
  if (not HasHookControl) then Exit;
     try
      Aux := FHookControl.ClassName;
     except
       on e : Exception do
       begin
         showmessage(e.message+' - '+TheMessage.msg.tostring);
         Exit;
       end;

     end;
 // debugln('TEmbedMessageHandler.SelfWndMethod : '+GetMessageName(TheMessage.msg)); //--- ct7777 -----

  case TheMessage.msg of
     LM_SHOWWINDOW,
     LM_ACTIVATE,
     LM_SETFOCUS,
     LM_MOVE,
     LM_SIZE,
     LM_ERASEBKGND,
     LM_PAINT,
     CM_SHOWINGCHANGED,
     CM_MENUCHANGED,
     CM_CONTROLLISTCHANGE //,
     //CM_TEXTCHANGED
                :begin
                  if (ehoSendBefore in Foptions) and Assigned(OnBeforeMessage)    then OnBeforeMessage(Self, FHookControl, TheMessage);
                  if (ehoSendToForm in Foptions) and Assigned(InheritedWndMethod) then InheritedWndMethod(TheMessage);
                  if (ehoSendAfter in Foptions)  and Assigned(OnAfterMessage)     then OnAfterMessage(Self, FHookControl, TheMessage);
                 end;
  else
     if (ehoSendToForm in Foptions) and Assigned(InheritedWndMethod) then InheritedWndMethod(TheMessage);
  end;

end;


//===========================================================================
//======================= TTEmbedFakeMainMenu ===============================
//===========================================================================

constructor TTEmbedFakeMainMenu.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  FMainMenu:=nil;
  Color:=clMenuBar;
  EdgeBorders:=[ebTop,ebBottom];
  ShowCaptions:=true;
  Align:=alBottom;
  List:=True;
end;

destructor TTEmbedFakeMainMenu.Destroy;
begin
  MainMenuClear;
  inherited Destroy;
end;

Procedure TTEmbedFakeMainMenu.MainMenuAssign(aMainMenu:TMainMenu);
begin
 FMainMenu:=aMainMenu;
 if FMainMenu=nil then exit;

 self.Images:=FMainMenu.Images;
end;

Procedure TTEmbedFakeMainMenu.MainMenuClear;
  var i:Integer;
begin

 for i := ButtonCount - 1 downto 0 do
   begin
     TToolButton(Buttons[i]).MenuItem:=nil;
     TToolButton(Buttons[i]).Free;
   end;

 ButtonList.Clear;
end;

Procedure TTEmbedFakeMainMenu.MainMenuUpdate;
  var i:Integer;
      xMenu    :TMenuItem;
      xButton  :TToolButton;

  Procedure _SetMenuItem(const val:integer);
  begin
      xMenu:=FMainMenu.Items[i];

      if xMenu<>nil then
       if xMenu.Visible then
       begin
        xButton:=TToolButton.Create(self);
        xButton.Parent:=self;
        xButton.Name:=xMenu.Name;
        xButton.Caption :=xMenu.Caption;
        xButton.MenuItem:=xMenu;
       end;
    end;

begin
if FMainMenu=nil then exit;

MainMenuClear;

if FMainMenu.IsRightToLeft=false then
begin
  for i:=FMainMenu.Items.Count-1 Downto 0 do _SetMenuItem(i);
end else
begin
  for i:=0 to FMainMenu.Items.Count-1     do _SetMenuItem(i);
end;

end;

//===========================================================================
//========================== TEmbedTabSheet =================================
//===========================================================================

constructor TEmbedTabSheet.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  FEditPages:=nil;
  FEditPageForSource:=nil;
  FEditPageForDesigner:=nil;
  FFormEditor:=nil;
  FSynEditor:=nil;
end;

destructor TEmbedTabSheet.Destroy;
begin
  inherited Destroy;
end;


procedure TEmbedTabSheet.EditPages_Create;
begin

  if FEditPages<>nil then exit;

  CT_EMBEDDED_UPDATING:=True;

  //-----------------
  FEditPages:= TPageControl.Create(self);
  FEditPages.Parent:=self;
  FEditPages.Align:=alClient;//alBottom;
  FEditPages.TabPosition:=tpBottom;
  FEditPages.ShowTabs:=CT_USE_EMBEDDED_DESIGNER;

  //--- page for Source code SynEdit ----
  FEditPageForSource:=FEditPages.AddTabSheet;
  FEditPageForSource.Name:='EditPageForSource1';
  FEditPageForSource.Caption:='Source';
  FEditPageForSource.TabVisible:=true;

  //--- page for FormDesigner -----------
  FEditPageForDesigner:=FEditPages.AddTabSheet;
  FEditPageForDesigner.Name:='EditPageForDesigner1';
  FEditPageForDesigner.Caption:='Designer';
  FEditPageForDesigner.TabVisible:=false;


  FEditPages.OnChange:=@DoOnTabControlChange;

  CT_EMBEDDED_UPDATING:=False;
end;

procedure TEmbedTabSheet.EditPages_Free;
begin
  if FEditPages=nil then exit;

  CT_EMBEDDED_UPDATING:=True;

  FormEditorDelete;
  FEditPages.Free;

  FEditPages:=nil;
  FEditPageForSource:=nil;
  FEditPageForDesigner:=nil;

  CT_EMBEDDED_UPDATING:=False;
end;

procedure TEmbedTabSheet.DoOnTabControlChange(Sender: TObject);
begin
  {$ifndef lclqt5}
  if CT_EMBEDDED_UPDATING then exit;
  {$endif}
  if FEditPages=nil then exit;

  if FEditPages.ActivePage=FEditPageForDesigner then
    if FFormEditor<>nil then
    begin
      FFormEditor.ActivatedDesigner;
      {$IFDEF LCLQT5}
       FFormEditor.sb_left.Click;
       FFormEditor.sb_zero.click;
      {$ENDIF}
    end;

end;

procedure TEmbedTabSheet.ShowSourceTab;
begin
  if CT_EMBEDDED_UPDATING then exit;

 if (FEditPages.ActivePage<>FEditPageForSource) then
     FEditPages.ActivePage:=FEditPageForSource;

 if FSynEditor<>nil then
  begin
    FSynEditor.SetFocus;
  end;
end;

procedure TEmbedTabSheet.ShowFormEditorTab;
 var xform:TCustomForm;
begin
{$IFNDEF LCLQT5}
 if CT_EMBEDDED_UPDATING then exit;
{$ENDIF}

 if FEditPages=nil then exit;

//------------------------------------- Check FFormEditor
 xform:=nil;

  if FFormEditor<>nil then
    if FFormEditor.FDesignPanel<>nil then
       if FFormEditor.FDesignPanel.FHasLoadedForm=false then
       begin
         xform:=FFormEditor.FDesignPanel.FLoadedForm;   // Get LoadedForm
         FormEditorDelete;                              // Delete OLD FormEditor

         if xform<>nil then
         begin

           if (CN_FORM_DESIGNER_PARENT_TYPE=ctptParentWindow) then
             xForm.ParentWindow:=0;                     // Reset LoadedForm ParentWindow

           xForm.Parent:=nil;                           // Reset LoadedForm Parent
           ShowWindow(xform.Handle, SW_HIDE);           // Hide  LoadedForm
           EmbeddedForm(nil,xform);                     // Embedded LoadedForm
         end;
       end;

//--------------------------------------
  if FEditPages=nil then exit;
  if FEditPageForDesigner.TabVisible then               // Only if Tab is visible for "Clone" Forms
   if (FEditPages.ActivePage<>FEditPageForDesigner) then
    begin

     FEditPages.ActivePage:=FEditPageForDesigner;
     if FFormEditor<>nil then FFormEditor.ActivatedDesigner;

    end;
end;

Procedure TEmbedTabSheet.UpdateEmbedFormDsgSettings;
begin
  if NOT Assigned(FFormEditor) then exit;

  CT_EMBEDDED_UPDATING:=True;

  FFormEditor.RootPanel.ParentColor:=false;
  FFormEditor.RootPanel.Color            := CT_USE_EMBEDDED_DESIGNER_BGCOLOR;

  FFormEditor.pnDesignBox.ParentColor:=false;
  FFormEditor.pnDesignBox.Color          := CT_USE_EMBEDDED_DESIGNER_BGCOLOR;

  FFormEditor.pnInfoPanel.Color          := CT_USE_EMBEDDED_INFOPANEL_COLOR;

  FFormEditor.pnScreenPreview.Color      := CT_USE_EMBEDDED_SRCEENPREVIEW_BGCOLOR;
  FFormEditor.pnScreenPreview.Visible    := CT_USE_EMBEDDED_SRCEENPREVIEW;

  FFormEditor.pnScreenPreviewBtns.Color  := CT_USE_EMBEDDED_DESIGNER_BGCOLOR;
  FFormEditor.pnScreenPreviewBtns.Visible:= CT_USE_EMBEDDED_SRCEENPREVIEW;

  CT_EMBEDDED_UPDATING:=False;
end;

procedure TEmbedTabSheet.FormEditorsInit;
 var R:TRect;
     SaltaCodigo : Boolean;
begin
  if CT_USE_EMBEDDED_DESIGNER=FALSE then Exit;

  CT_EMBEDDED_UPDATING:=True;
  if FFormEditor<>nil then FFormEditor.Free else
  SaltaCodigo := True;

  FFormEditor:=TEmbedFormEditor.Create(FEditPageForDesigner);
  FFormEditor.Parent:=FEditPageForDesigner;
  FFormEditor.Visible:=false;
  FFormEditor.Align:=alClient;

  //... Set Images.......
  IDEImages.AssignImage(FFormEditor.sb_up,      'ct_moveu');
  IDEImages.AssignImage(FFormEditor.sb_bottom,  'ct_moved');
  IDEImages.AssignImage(FFormEditor.sb_left,    'ct_movel');
  IDEImages.AssignImage(FFormEditor.sb_right,   'ct_mover');
  IDEImages.AssignImage(FFormEditor.sb_zero,    'ct_reset');
  IDEImages.AssignImage(FFormEditor.sb_update,  'ct_update');
  IDEImages.AssignImage(FFormEditor.sb_hide,    'ct_show');

  //... Set Hint ........
  FFormEditor.sb_up.Hint          :=lisEFDup;
  FFormEditor.sb_bottom.Hint      :=lisEFDbottom;
  FFormEditor.sb_left.Hint        :=lisEFDleft;
  FFormEditor.sb_right.Hint       :=lisEFDright;
  FFormEditor.sb_zero.Hint        :=lisEFDzero;
  FFormEditor.sb_update.Hint      :=lisEFDupdate;
  FFormEditor.sb_hide.Hint        :=lisEFDhide;
  FFormEditor.pnScreenPreview.Hint:=lisEFDScreenPreview;

  //........... Info Panel .......................... ct2222
  FFormEditor.pnInfoPanel.Visible:=False;
  FFormEditor.pnInfoPanel.SetBounds(FFormEditor.Width-FFormEditor.pnInfoPanel.Width-(LCLIntf.GetSystemMetrics(SM_CYVSCROLL)+4),
                                    4,
                                    FFormEditor.pnInfoPanel.Width,FFormEditor.pnInfoPanel.Height);

  //........... Screen Preview Panel.....................
  R:=Rect(FFormEditor.pnScreenPreview.Left,
          FFormEditor.pnScreenPreview.top,
          FFormEditor.PixelPerc(Screen.Width,  CN_PREVIEW_FACTOR),
          FFormEditor.PixelPerc(Screen.Height, CN_PREVIEW_FACTOR)
          );
  R.Left:=FFormEditor.Width -(R.Right +LCLIntf.GetSystemMetrics(SM_CYVSCROLL)+4);
  R.Top :=FFormEditor.Height-(R.Bottom+LCLIntf.GetSystemMetrics(SM_CXHSCROLL)+4);

  FFormEditor.pnScreenPreview.SetBounds(R.Left, R.top, R.Right, R.Bottom);

  //.......... Show Hide Panel ..........................
  R:=Rect(FFormEditor.pnShowHide.Left,
          FFormEditor.pnShowHide.top,
          FFormEditor.pnShowHide.Width,
          FFormEditor.pnShowHide.Height
          );

  R.Left:=FFormEditor.pnScreenPreview.Left + (FFormEditor.pnScreenPreview.Width-R.Right);
  R.Top :=FFormEditor.pnScreenPreview.Top  -  FFormEditor.pnShowHide.Height - 4;

  FFormEditor.pnShowHide.SetBounds(R.Left, R.top, R.Right, R.Bottom);

   //.......... Screen Preview Buttons Panel .............
  R:=Rect(FFormEditor.pnScreenPreviewBtns.Left,
          FFormEditor.pnScreenPreviewBtns.top,
          FFormEditor.pnScreenPreviewBtns.Width,
          FFormEditor.pnScreenPreviewBtns.Height
          );
  R.Left:=FFormEditor.pnScreenPreview.Left + (FFormEditor.pnScreenPreview.Width-R.Right-FFormEditor.pnShowHide.Width-4);
  R.Top :=FFormEditor.pnScreenPreview.Top  -  FFormEditor.pnScreenPreviewBtns.Height - 4;

  FFormEditor.pnScreenPreviewBtns.SetBounds(R.Left, R.top, R.Right, R.Bottom);


  //........... Design Panel .......................
  FFormEditor.FDesignPanel := TEmbedDesignPanel.Create(FFormEditor.pnDesignBox);
  FFormEditor.FDesignPanel.Parent := FFormEditor.pnDesignBox;
  FFormEditor.FDesignPanel.Visible:=false;
  FFormEditor.FDesignPanel.dsParentPage:=self;
  FFormEditor.FDesignPanel.dsParentFormEditor:=FFormEditor;
  FFormEditor.FDesignPanel.OnFormLoad := @FFormEditor.DesignPanel_OnLoadForm;
  FFormEditor.FDesignPanel.OnLoadedFormChangeBounds := @FFormEditor.DesignPanel_OnLoadedFormChangeBounds;

  UpdateEmbedFormDsgSettings;

  FEditPageForDesigner.TabVisible:=true;
  {$IFDEF LCLQT5}
  if FEditPages=nil then exit;
  if (FFormEditor.FDesignPanel.LoadedForm is Tform) then
  begin
  if FEditPageForDesigner.TabVisible then               // Only if Tab is visible for "Clone" Forms
   if (FEditPages.ActivePage<>FEditPageForDesigner) then
    begin

     FEditPages.ActivePage:=FEditPageForDesigner;
     if FFormEditor<>nil then FFormEditor.ActivatedDesigner;

    end;
  end else
  begin
    if not Saltacodigo then
     begin
      Application.ProcessMessages;
      FEditPages.ActivePage:=FEditPageForDesigner;
     end;

  end;
  {$ENDIF}



  CT_EMBEDDED_UPDATING:=False;
end;

Procedure TEmbedTabSheet.FormEditorDelete;
begin
if CT_USE_EMBEDDED_DESIGNER=FALSE then Exit;

 if FFormEditor<>nil then FFormEditor.free;
 FFormEditor:=nil;

 if FEditPageForDesigner<> nil then FEditPageForDesigner.TabVisible:=false;
end;

Procedure TEmbedTabSheet.FormEditorAdd;
begin
 if CT_USE_EMBEDDED_DESIGNER=FALSE then Exit;
 if FEditPageForDesigner<> nil then FEditPageForDesigner.TabVisible:=true;
end;


Function TEmbedTabSheet.GetLoadedForm:TCustomForm;
begin
 Result:=nil;

 if FFormEditor<>nil then
   if FFormEditor.FDesignPanel<>nil then
     if FFormEditor.FDesignPanel.LoadedForm<>nil then
        Result:=FFormEditor.FDesignPanel.LoadedForm;
end;











Procedure TEmbedTabSheet.EmbeddedForm(Sender: TObject; aForm: TCustomForm);
begin
  if FEditPages=nil then exit;
  if FFormEditor<>nil then exit;     // if has form then exit...

  FormEditorsInit;

  if FFormEditor=nil then exit;
  if FFormEditor.FDesignPanel=nil then exit;

  CT_EMBEDDED_UPDATING:=True;

  FFormEditor.FDesignPanel.EmbeddedForm(Sender,aForm);

  if FEditPageForDesigner<> nil then FEditPageForDesigner.TabVisible:=true;

  FFormEditor.FDesignPanel.IniOK:=false;
  FFormEditor.ActivatedDesigner;
  //saulo
  {$IFDEF LCLQT5}
  FFormEditor.FDesignPanel.PaintGuides(FFormEditor.FDesignPanel);
  {$ENDIF}
  CT_EMBEDDED_UPDATING:=False;

  if aForm=nil then
    DebugLn(EMBEDDED_FORM_DESIGNER,'TEmbedTabSheet.EmbeddedForm : NIL') else
    DebugLn(EMBEDDED_FORM_DESIGNER,'TEmbedTabSheet.EmbeddedForm :'+aForm.Name);
end;


//================================================================== SOS SOS SOS
// This Is for Problem of "Docking Embedded Form Designer"
// The GlassDocking send LM_DESTROY msg to FLoadedForm
// On Windows give Error 1400 and crash the IDE
// On GTK2    give Error and crash the IDE
// We don't know why, must find this and remove the next code...
//==================================================================

Procedure TEmbedTabSheet.FormEditorPause;
begin

  if CT_USE_EMBEDDED_DESIGNER=FALSE then Exit;
  if FEditPages=nil then exit;
  if FFormEditor=nil then exit;     // if has form then exit...
  if FFormEditor.FDesignPanel=nil then exit;
  if FFormEditor.FDesignPanel.FHasLoadedForm=false then exit;

  DebugLn(EMBEDDED_FORM_DESIGNER,'TEmbedTabSheet.FormEditorPause START');

 CT_EMBEDDED_UPDATING:=True;

//--- ct5555 --------------------
{$IF (NOT DEFINED(LCLGTK2))}
// Must on LCLWin32, LCLWin64
// Must NOT on LCLGTK2
  if (CN_FORM_DESIGNER_PARENT_TYPE=ctptParentWindow) then
      FFormEditor.FDesignPanel.FLoadedForm.Parent:=FFormEditor.FDesignPanel;

{$ENDIF}
//------------------------------

  FFormEditor.FDesignPanel.FMessagesHandler.UnHookControl;
  FFormEditor.FDesignPanel.FHasLoadedForm := False;


  if FEditPageForDesigner<> nil then FEditPageForDesigner.TabVisible:=false;
  CT_EMBEDDED_UPDATING:=False;
end;

//====================================

initialization
  EMBEDDED_FORM_DESIGNER := DebugLogger.RegisterLogGroup('EMBEDDED_FORM_DESIGNER' {$IFDEF EMBEDDED_FORM_DESIGNER} , True {$ENDIF} );

end.


